# !/usr/bin/env python

import glob
import json
from json import JSONDecodeError
import os
import pandas as pd
import sys
import time

sys.path.append('..')

from voyllect.save import save_data
from voyllect.utils import n_pages


def aggregate_new_urls(source_dict, 
                       new_urls_folder_path, 
                       aggregated_urls_folder_path):
    """Aggregates new URLs in 'aggregated_urls' folder.

    Args:
        source_dict (dict): dictionary with information from the source.
        new_urls_folder_path (str): path to the 'new_urls' folder.
        aggregated_urls_folder_path (str): path to the 'aggregated_urls' folder.
    """

    print("[LOG] Start to aggregate new URLs.")

    # Load and aggregate the new URLs
    new_urls_dicts = []
    for new_url_dicts_file in glob.glob(os.path.join(new_urls_folder_path, '*.json')):
        for new_url_dict in json.load(open(new_url_dicts_file, 'r', encoding='utf8')):
            new_urls_dicts.append(new_url_dict)

    # Save the aggregated new URLs in 'aggregated_urls' folder
    save_data(data=new_urls_dicts, 
              saved_data_type='aggregated_urls', 
              source=source_dict['source'], 
              path=aggregated_urls_folder_path)
    
    print(f"[LOG] {len(new_urls_dicts)} aggregated URLs have been saved "
           "in 'aggregated_urls' folder.")
    print("[LOG] The new URLs files have been aggregated.")


def remove_duplicates(dicts, 
                      key):
    """Removes duplicates from a list of dictionaries based on 
    a specified key.

    Args:
        dicts (list[dict]): list of dictionaries.
        key (str): key to use for comparing duplicates.

    Returns:
        list[dict], List of dictionaries with duplicates removed 
                    based on the specified key.
    """

    unique_keys = set()
    removed_duplicates_dicts = []

    for d in dicts:
        if d[key] not in unique_keys:
            unique_keys.add(d[key])
            removed_duplicates_dicts.append(d)

    return removed_duplicates_dicts


def remove_elements_with_keywords(dicts, 
                                  keys, 
                                  keywords):
    """Removes elements from a list of dictionaries if a 
    specific key contains a specific word.

    Args:
        dicts (list[dict]): a list of dictionaries.
        keys (list): list of keys to check for the keyword.
        keywords (list): list of words to look for in the value of the key.

    Returns:
        list[dict], List of dictionaries with elements removed based on the 
                    presence of the keywords.
    """

    results = []
    for d in dicts:
        keep = True
        for key in keys:
            if key not in d:
                continue
            value = str(d[key]).lower().strip()
            for keyword in keywords:
                if str(keyword).lower().strip() in value:
                    keep = False
                    break
            if not keep:
                break
        if keep:
            results.append(d)

    return results


def select_elements_with_keywords(dicts, 
                                  keys, 
                                  keywords):
    """Selects elements from a list of dictionaries if 
    a specific key contains a specific word.

    Args:
        dicts (list[dict]): A list of dictionaries.
        keys (list): List of keys to check for the keyword.
        keywords (list): List of words to look for in the value of the key.

    Returns:
        list[dict], List of dictionaries with elements selected based on the 
                    presence of the keywords.
    """

    results = []
    for d in dicts:
        keep = False
        for key in keys:
            if key not in d:
                continue
            value = str(d[key]).lower().strip()
            for keyword in keywords:
                if str(keyword).lower().strip() in value:
                    keep = True
                    break
            if keep:
                break
        if keep:
            results.append(d)

    return results


def filter_urls(source_dict, 
                keywords_for_removing, 
                keywords_for_selecting,
                new_urls_folder_path, 
                filtered_urls_folder_path):
    """Filters new URLs in 'filtered_urls' folder.

    Args:
        source_dict (dict): dictionary with information from the source.
        keywords_for_removing (list): list of keywords for removing. 
        keywords_for_selecting (list): list of keywords for selecting.
        new_urls_folder_path (str): path to the 'new_urls' folder.
        filtered_urls_folder_path (str): path to the 'filtered_urls' folder.
    """

    print("[LOG] Start to filter new URLs.")

    # Load and aggregate the new URLs
    new_urls_dicts = []
    for new_url_dicts_file in glob.glob(os.path.join(new_urls_folder_path, '*.json')):
        for new_url_dict in json.load(open(new_url_dicts_file, 'r', encoding='utf8')):
            new_urls_dicts.append(new_url_dict)

    # Filter the new URLs
    # Remove the duplicates
    filtered_urls_dicts = remove_duplicates(dicts=new_urls_dicts, 
                                            key='url')
    
    # Remove the elements with specific keywords
    if keywords_for_removing:
        filtered_urls_dicts = \
            remove_elements_with_keywords(dicts=filtered_urls_dicts, 
                                          keys=['product_name', 'product_brand', 'url'], 
                                          keywords=keywords_for_removing)
        
    # Select the elements with specific keywords
    if keywords_for_selecting:
        filtered_urls_dicts = \
            select_elements_with_keywords(dicts=filtered_urls_dicts, 
                                          keys=['product_name', 'product_brand', 'url'], 
                                          keywords=keywords_for_selecting)
        
    # Remove the duplicates
    filtered_urls_dicts = remove_duplicates(dicts=filtered_urls_dicts, 
                                            key='url')

    # Save filtered URLs in 'filtered_urls' folder
    save_data(data=filtered_urls_dicts,
              saved_data_type='filtered_urls',
              source=source_dict['source'],
              path=filtered_urls_folder_path)
    
    print(f"[LOG] {len(filtered_urls_dicts)} filtered URLs have been saved "
           "in 'filtered_urls' folder.")
    print("[LOG] The new URLs files have been filtered.")


def generate_urls_to_collect_dicts(filtered_urls_dicts):
    """Generates a list of product URLs to collect dictionaries
    from a list of new product URLs dictionaries.

    Args:
        filtered_urls_dicts (list[dict]): lst of dictionaries containing new product URLs.

    Returns:
        list[dict], List of dictionaries containing the product URLs, 
                    their category, and whether they have been collected.
    """

    urls_to_collect_dicts = [
        {
            'url': u['url'], 
            'collected': 'no'
        }
        for u in filtered_urls_dicts
    ]
    
    return urls_to_collect_dicts


def generate_urls_to_collect(source_dict, 
                             n_parts,
                             filtered_urls_dicts_object_name, 
                             urls_to_collect_folder_path, 
                             urls_to_collect_anchor_folder_path):
    """Generated URLs to collect files.

    Args:
        source_dict (dict): dictionary with information from the source.
        n_parts (int): number of partitions for the urls to collect files.
        filtered_urls_dicts_object_name (str): object name of the filtered URLs.
        urls_to_collect_folder_path (str): path to the 'urls_to_collect' folder.
        urls_to_collect_anchor_folder_path (str): path to the 'urls_to_collect_anchor' folder.
    """ 

    print("[LOG] Start to generate URLs to collect.")
    print(f"[LOG] Filtered URLs object name: {filtered_urls_dicts_object_name}.")
   
    # Load the filtered URLs
    with open(os.path.join(filtered_urls_dicts_object_name),
              encoding='utf-8') as file_to_open:
        filtered_urls_dicts = json.load(file_to_open)
    print(f"[LOG] There are {len(filtered_urls_dicts)} filtered URLs.")

    # Generate the URLs to collect
    urls_to_collect_dicts = \
        generate_urls_to_collect_dicts(filtered_urls_dicts=filtered_urls_dicts)
    
    # Calculate the size of each partition
    partition_size = len(urls_to_collect_dicts) // n_parts

    # Split the URLs to collect in partitions
    for i in range(0, len(urls_to_collect_dicts), partition_size):
        tmp_urls_to_collect_dicts = urls_to_collect_dicts[i:i + partition_size]

        # Save URLs to collect in 'urls_to_collect' folder
        save_data(data=tmp_urls_to_collect_dicts,
                  saved_data_type='urls_to_collect',
                  source=source_dict['source'],
                  path=urls_to_collect_folder_path)

        # Save URLs to collect in 'urls_to_collect_anchor' folder
        save_data(data=tmp_urls_to_collect_dicts,
                  saved_data_type='urls_to_collect_anchor',
                  source=source_dict['source'],
                  path=urls_to_collect_anchor_folder_path)
        
        # Add 2 second tempo for the unicity in the file name
        time.sleep(2)

        print(f"[LOG] {len(tmp_urls_to_collect_dicts)} URLs to collect dictionaries have been saved "
               "in 'urls_to_collect' and 'urls_to_collect_anchor' folders.")
    
    if n_parts == 1:
        print("[LOG] The URLs to collect file have been generated.")
    elif n_parts > 1:
        print("[LOG] The URLs to collect files have been generated.")


def generate_product_url_to_collect_dict(product_url):
    """Generates product URL to collect dictionary.
    
    Args:
        product_url (str): product URL.

    Returns:
        dict, Product URL to collect dictionary.
    """

    return {
        'url': str(product_url),
        'collected': 'no'
    }


def generate_reviews_urls_to_collect_dicts(product_url, 
                                           n_reviews, 
                                           n_reviews_per_page,
                                           str_prefix,
                                           str_between_url_n_pages,
                                           str_suffix):
    """Generates reviews URLs to collect dictionaries.

    Args:
        product_url (str): product URL.
        n_reviews (int): number of reviews.
        n_reviews_per_page (int): number of reviews per page.
        str_prefix (str): string to add at the beginning of the reviews URL.
        str_between_url_n_pages (str): string to add between the 
                                       reviews URL and the number of pages.
        str_suffix (str): string to add at the end of the reviews URL.

    Returns:
        list[dict], List of reviews URLs to collect dictionaries.
    """
    
    reviews_urls_to_collect_dicts = []

    if n_reviews_per_page:
        for n_page in range(1, n_pages(n_reviews=n_reviews,
                                       n_reviews_per_page=n_reviews_per_page)):
            url = str(product_url)
            if str_prefix:
                url = str_prefix + str(product_url)
            if str_between_url_n_pages:
                url = url + str_between_url_n_pages + str(n_page)
            if str_suffix:
                url = url + str_suffix

            reviews_urls_to_collect_dicts.append({
                'url': url,
                'collected': 'no'
            })
    else:
        print("[LOG] The value `n_reviews_per_page` is not defined.")
    
    return reviews_urls_to_collect_dicts


def aggregate_products_files(source_dict,
                             products_folder_path, 
                             aggregated_products_folder_path):
    """Aggregates the collected products files.
    
    Args:
        source_dict (dict): dictionary with information from the source.
        products_folder_path (str): path to the products data files folder.
        aggregated_products_folder_path (str): path to the aggregated products folder.
    
    Returns:
        list[dict], aggregated list of product dictionaries.
    """

    print("[LOG] Start to aggregate products files.")
    
    products_files = []
    
    for products_file in glob.glob(os.path.join(products_folder_path, '*.json')):
        try:
            with open(products_file, 'r', encoding='utf8') as f:
                open_product_file = json.load(f)
                products_files.append(open_product_file)
        except JSONDecodeError:
            pass

    aggregated_products_file_name = os.path.join(
        aggregated_products_folder_path, 
        f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_aggregated_products_{source_dict['source']}.json")
    
    with open(aggregated_products_file_name, 'w', encoding='utf-8') as file_to_dump:
        json.dump(products_files, file_to_dump, indent=4, ensure_ascii=False)
    
    print(f"[LOG] There are {len(products_files)} aggregated products.")
    print("[LOG] The products files have been aggregated.")

    return products_files


def aggregate_reviews_files(source_dict,
                            reviews_folder_path,
                            aggregated_reviews_folder_path):
    """Aggregates the collected reviews files.

    Args:
        source_dict (dict): dictionary with information from the source.
        reviews_folder_path (str): path to the reviews data files folder.
        aggregated_reviews_folder_path (str): path to the aggregated reviews folder.
        
    Returns:
        list[dict], aggregated list of reviews dictionaries.
    """

    print("[LOG] Start to aggregate reviews files.")

    reviews_files = []

    for reviews_file in glob.glob(os.path.join(reviews_folder_path, '*.json')):
        try:
            with open(reviews_file, 'r', encoding='utf8') as f:
                reviews_dicts = json.load(f)
                reviews_files.extend(reviews_dicts)
        except JSONDecodeError:
            pass

    aggregated_reviews_file_name = os.path.join(
        aggregated_reviews_folder_path,
        f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_aggregated_reviews_{source_dict['source']}.json") 

    with open(aggregated_reviews_file_name, 'w', encoding='utf-8') as file_to_dump:
        json.dump(reviews_files, file_to_dump, indent=4, ensure_ascii=False)

    print(f"[LOG] There are {len(reviews_files)} aggregated reviews.")
    print("[LOG] The reviews files have been aggregated.")

    return reviews_files


def select_not_dict_columns(aggregated_data):
    """ Filter out the dictionary attributes from the aggregated data file.

    Args:
        aggregated_data (list[dict]): Aggregated data file.
    
    Returns:
        list, list of attributes that are not dictionaries.
    """

    not_dict_columns = []

    for aggregated_data_element in aggregated_data:
        for key in aggregated_data_element.keys():
            if type(aggregated_data_element[key]) != dict:
                not_dict_columns.append(key)

    not_dict_columns_set = set(not_dict_columns)

    return list(not_dict_columns_set)


def build_counts_per_urls_kpis(aggregated_products, 
                               aggregated_reviews, 
                               source_dict, 
                               kpis_counts_per_urls_folder_path):
    """Creates KPIs EXCEL files for collect analysis.

    Args:
        aggregated_products (list[dict]): list of aggregated products dictionaries.
        aggregated_reviews (list[dict]): list of aggregated reviews dictionaries.
        source_dict (dict): dictionary with information from the source.
        kpis_counts_per_urls_folder_path (str): path to the counts per urls KPIs folder.
    
    Returns:
        df, Dataframe of KPIs.
    """
        
    reviews_df = \
        pd.DataFrame(aggregated_reviews)[['url', 'review_title', 'review_text', 'review_rating', 'review_date']].\
            drop_duplicates()

    reviews_value_counts_df = pd.DataFrame(reviews_df.url.value_counts()).reset_index()
    reviews_value_counts_df.columns = ['url', 'n_saved_reviews']

    products_df = pd.DataFrame(aggregated_products)
    products_df_columns = products_df.columns
    if 'n_review_only' and 'n_ratings_only' in products_df_columns:
        products_df = \
            products_df[
                ['url', 'product_name', 'product_brand', 'n_reviews', 'n_reviews_only', 'n_ratings_only']]
        products_df = products_df.dropna().drop_duplicates().reset_index(drop=True)
    else:
        products_df = \
            products_df[['url', 'product_name', 'product_brand', 'n_reviews']]
        products_df = products_df.dropna().drop_duplicates().reset_index(drop=True)

    kpis_df = pd.merge(products_df, reviews_value_counts_df, on='url', how='outer')
    kpis_df = kpis_df.drop_duplicates().reset_index(drop=True)

    kpis_df.to_excel(
        os.path.join(
            kpis_counts_per_urls_folder_path,
              f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_aggregated_products_and_reviews_kpis_{source_dict['source']}.xlsx"),
                index=False)
    
    return kpis_df
    

def build_counts_per_collect_date_kpis(aggregated_products, 
                                       aggregated_reviews, 
                                       source_dict, 
                                       kpis_counts_per_collect_date_products_folder_path,
                                       kpis_counts_per_collect_date_reviews_folder_path):
    """Creates KPIs EXCEL files for collect speed analysis.

    Args:
        aggregated_products (list[dict]): list of aggregated products dictionaries.
        aggregated_reviews (list[dict]): list of aggregated reviews dictionaries.
        source_dict (dict): dictionary with information from the source.
        kpis_counts_per_urls_products_folder_path (str): path to the products counts per collect date KPIs folder.
        kpis_counts_per_urls_reviews_folder_path (str): path to the reviews counts per collect_date KPIs folder.
    
    Returns:
        df, Dataframe of KPIs.
    """

    products_df = pd.DataFrame(aggregated_products)
    products_value_counts_df = pd.DataFrame(products_df.collect_date.value_counts()).reset_index()
    products_value_counts_df.columns = ['collect_date', 'n_saved_products']
    products_value_counts_df.sort_values(by='n_saved_products', ascending=False, inplace=True)

    reviews_df = pd.DataFrame(aggregated_reviews)
    reviews_value_counts_df = pd.DataFrame(reviews_df.collect_date.value_counts()).reset_index()
    reviews_value_counts_df.columns = ['collect_date', 'n_saved_reviews']
    reviews_value_counts_df.sort_values(by='n_saved_reviews', ascending=False, inplace=True)

    products_value_counts_df.to_excel(
        os.path.join(
            kpis_counts_per_collect_date_products_folder_path,
                f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_collect_products_speed_kpis_{source_dict['source']}.xlsx"),
                    index=False)
    
    reviews_value_counts_df.to_excel(
        os.path.join(
            kpis_counts_per_collect_date_reviews_folder_path,
                f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_collect_reviews_speed_kpis_{source_dict['source']}.xlsx"),
                    index=False)
    
    return products_value_counts_df, reviews_value_counts_df
    

def build_samples_kpis(aggregated_products, 
                       aggregated_reviews, 
                       source_dict,
                       kpis_samples_products_folder_path,
                       kpis_samples_reviews_folder_path):
    """Creates KPIs EXCEL files with sample from the data.

    Args:
        aggregated_products (list[dict]): list of aggregated products dictionaries.
        aggregated_reviews (list[dict]): list of aggregated reviews dictionaries.
        source_dict (dict): dictionary with information from the source.
        kpis_samples_products_folder_path (str): path to the products sample KPIs folder.
        kpis_samples_reviews_folder_path (str): path to the reviews sample KPIs folder.
    
    Returns:
        df, Dataframe of KPIs.
    """

    # Sample size
    products_sample_size = min(len(aggregated_products), 15)
    reviews_sample_size = min(len(aggregated_reviews), 15)

    # Sample for the products data collected
    aggregated_products_df = pd.DataFrame(aggregated_products)
    aggregated_products_sample_df = aggregated_products_df.sample(n=products_sample_size, random_state=1)

    # Sample for the reviews data collected
    aggregated_reviews_df = pd.DataFrame(aggregated_reviews)
    aggregated_reviews_sample_df = pd.DataFrame(aggregated_reviews_df).sample(n=reviews_sample_size, random_state=1)
    
    aggregated_products_sample_df.to_excel(
        os.path.join(
            kpis_samples_products_folder_path,
              f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_data_sample_products_kpis_{source_dict['source']}.xlsx"),
                index=False)

    aggregated_reviews_sample_df.to_excel(
        os.path.join(
            kpis_samples_reviews_folder_path,
              f"{time.strftime('%Y_%m_%d_%H_%M_%S')}_data_sample_reviews_kpis_{source_dict['source']}.xlsx"),
                index=False)
    
    return aggregated_reviews_sample_df, aggregated_reviews_sample_df
    


#!/usr/bin/env python

import sys
import random

sys.path.append('..')

from fake_useragent import UserAgent as ua


def get_random_user_agent(browsers=['Chrome'], min_version=120.0):
    """Generates random user agent.
    
    Args:
        browsers (list): list of web browsers.
        min_version (float): minimum version of the selected web browsers.

    Returns:
        str, Random user agent.
    s"""

    user_agents = ua(browsers=browsers, min_version=min_version)

    return user_agents.random


def get_driver_dict(driver_path, 
                    options, 
                    args):
    """Gets the driver parameters dictionary.
    
    Args:
        driver_path (str): path to the driver.
        options (dict): options of the driver.
        args (dict): parsed arguments.

    Returns:
        dict, Driver parameters dictionary.
    """

    return {
        'driver_path': driver_path,
        'options': options,
        'headless': args.headless,
        'delete_cookies': args.delete_cookies,
        'use_driver': args.use_driver,
        'use_proxy': args.use_proxy,
        'proxy_url': args.proxy_url
    }


def quit_driver(driver, 
                delete_cookies):
    """Quit the driver.

    Args:
        driver (WebDriver): selenium webdriver.
        delete_cookies (bool): to delete cookies or not.
    """
    
    try:
        driver.quit()
        if delete_cookies:
            driver.delete_all_cookies()
    except Exception as e:
        print(f"[LOG] [EXCEPTION]\n{e}")
        


#!/usr/bin/env python

import glob
import os
import sys

sys.path.append('..')

from math import ceil


def convert_n_reviews_to_int(n_reviews):
    """Converts the `n_reviews` to int.
    
    Args:
        n_reviews: value of reviews count.
        
    Return:
        int, Reviews count in integer.
    """

    if isinstance(n_reviews, int):
        return n_reviews
    elif isinstance(n_reviews, float):
        return int(n_reviews)
    else:
        try:
            # Remove non-numeric characters
            n_reviews = ''.join(filter(str.isdigit, n_reviews))  
            return int(n_reviews)
        except Exception as e:
            print(f"[LOG] [EXCEPTION]\n{e}")
            return 0


def check_n_reviews_limit(n_reviews, n_reviews_max):
    """Compare number of reviews loaded/saved to the limit 'n_reviews_max'.
    
    Args:
        n_reviews: value of reviews count.
        n_reviews_max: limit of reviews to load/save.
        
    Return:
        Bool, if the limit has been exceeded or not.
    """

    if n_reviews >= n_reviews_max:
        print("[LOG] [LIMIT] The reviews limit has been reached.")
        return True
    else:
        return False


def get_rating_from_colors(colors):
    """Evaluates the rating value based on the number of occurrences 
    of the color "#3cbeaf" in a list of colors.

    Args:
        colors (list): list of color codes in string 
                       format, such as "#3cbeaf" or "#E6E6E6".

    Returns:
        int, The rating value corresponding to the number of consecutive 
             occurrences of the color "#3cbeaf" in the list.
             The rating value starts at 1 and increases by 1 for 
             each group of consecutive "#3cbeaf" colors.
    """

    return colors.count("#3cbeaf")


def get_most_recent_json_file(folder_path):
    """Returns path to the most recent json file in a the specified folder.

    Args:
        folder_path (str): path of folder containing json files.

    Returns:
        str, most recently created json file path.
    """
    
    most_recent_json_file = \
        sorted(glob.glob(os.path.join(folder_path, '*.json')))[-1]
    
    return most_recent_json_file


def n_pages(n_reviews, n_reviews_per_page):
    """Determines the number of reviews pages per product.

    Args:
        n_reviews (int): number of reviews.
        n_reviews_per_page (int): number of reviews per page.

    Returns:
        int, Number of reviews pages.
    """

    return ceil(int(n_reviews) / n_reviews_per_page) + 1


def delete_files_after_transfer(aggregated_urls_folder_path,
                                aggregated_urls,
                                filtered_urls_folder_path,
                                filtered_urls,
                                reviews_folder_path,
                                reviews,
                                products_folder_path,
                                products,
                                urls_to_collect_anchor_folder_path,
                                urls_to_collect_anchor,
                                aggregated_products_folder_path,
                                aggregated_products,
                                aggregated_reviews_folder_path,
                                aggregated_reviews,
                                counts_per_urls_folder_path,
                                counts_per_urls,
                                counts_per_collect_date_products_folder_path,
                                counts_per_collect_date_products,
                                counts_per_collect_date_reviews_folder_path,
                                counts_per_collect_date_reviews,
                                samples_products_folder_path,
                                samples_products,
                                samples_reviews_folder_path,
                                samples_reviews):
    """Deletes files after ransferrind data to S3.

    Args:
        aggregated_urls_folder_path (str): folder path to aggregated URLs.
        aggregated_urls (bool): delete aggregated URLs.
        filtered_urls_folder_path (str): folder path to filtered URLs.
        filtered_urls (bool): delete filtered URLs.
        reviews_folder_path (str): folder path to the reviews.
        reviews (bool): delete reviews files.
        products_folder_path (str): folder path to the products.
        products (bool): delete products files.
        aggregated_products_folder_path (str): folder path to aggregated products.
        aggregated_products (bool): delete aggregated products.
        aggregated_reviews_folder_path (str): folder path to aggregated reviews.
        aggregated_reviews (bool): delete aggregated reviews.
        counts_per_urls_folder_path (str): folder path to KPIs files.
        counts_per_urls (bool): delete KPIs files.
        counts_per_collect_date_products_folder_path (str): folder path to the products KPIs files.
        counts_per_collect_date_products (bool): delete KPIs files.
        counts_per_collect_date_reviews_folder_path (str): folder path to the reviews KPIs files.
        counts_per_collect_date_reviews (bool): delete KPIs files.
        samples_products_folder_path (str): folder path to the samples products KPIs files.
        samples_products (bool): delete KPIs files.
        samples_reviews_folder_path (str): folder path to the samples reviews KPIs files.
        samples_reviews (bool): delete KPIs files.
    """

    print("[LOG] Deleting files that have been transfered to S3...")
    confirm_transfer = input("[LOG] Confirm transfer (yes/no) : ")
    confirm_operation = input("[LOG] Confirm operation (yes/no) : ")

    if confirm_transfer == 'yes' and confirm_operation ==  'yes':
        # reviews
        if reviews:
            print(f"[LOG] Deleting content of : {reviews_folder_path} ...")
            files = glob.glob(os.path.join(reviews, '*.json'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # products
        if products:
            print(f"[LOG] Deleting content of : {products} ...")
            files = glob.glob(os.path.join(products, '*.json'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # Filtered URLs
        if filtered_urls:
            print(f"[LOG] Deleting content of : {filtered_urls_folder_path} ...")
            files = glob.glob(os.path.join(filtered_urls_folder_path, '*.json'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # Aggregated products
        if aggregated_products:
            print(f"[LOG] Deleting content of : {aggregated_products_folder_path} ...")
            files = glob.glob(os.path.join(aggregated_products_folder_path, '*.json'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # Aggregated reviews
        if aggregated_reviews:
            print(f"[LOG] Deleting content of : {aggregated_reviews_folder_path} ...")
            files = glob.glob(os.path.join(aggregated_reviews_folder_path, '*.json'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # counts_per_urls
        if counts_per_urls:
            print(f"[LOG] Deleting content of : {counts_per_urls_folder_path} ...")
            files = glob.glob(os.path.join(counts_per_urls_folder_path, '*.xlsx'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # counts_per_collect_date_products
        if counts_per_collect_date_products:
            print(f"[LOG] Deleting content of : {counts_per_collect_date_products_folder_path} ...")
            files = glob.glob(os.path.join(counts_per_collect_date_products_folder_path, '*.xlsx'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # counts_per_collect_date_reviews
        if counts_per_collect_date_reviews:
            print(f"[LOG] Deleting content of : {counts_per_collect_date_reviews_folder_path} ...")
            files = glob.glob(os.path.join(counts_per_collect_date_reviews_folder_path, '*.xlsx'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # samples_products
        if samples_products:
            print(f"[LOG] Deleting content of : {samples_products_folder_path} ...")
            files = glob.glob(os.path.join(samples_products_folder_path, '*.xlsx'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")

        # samples_reviews
        if samples_reviews:
            print(f"[LOG] Deleting content of : {samples_reviews_folder_path} ...")
            files = glob.glob(os.path.join(samples_reviews_folder_path, '*.xlsx'))
            for file in files:
                os.remove(file)
            print("[LOG] Files deleted.")
    else:
        print("[LOG] Operation cancelled.")



#!/usr/bin/env python

import argparse
import os
import sys
from distutils.util import strtobool

sys.path.append('..')

from voyllect.utils import get_most_recent_json_file


def str_to_bool(string_input):
    """Converts the command line string to bool.
    
    Args:
        string_input (str): string input in the command line.
    
    Returns:
        bool, Boolean value of the argument.
    """

    try:
        return bool(strtobool(string_input))
    except ValueError:
        raise argparse.ArgumentTypeError(\
            "[LOG] [COMMAND LINE] Invalid value for boolean flag.")


def create_products_listing_pages_files_arg_parser():
    """Provides a parser to parse arguments for the 
    create_products_listing_pages_files.py script.

    Returns:
        namespace, Parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Creates products-listing pages files.")

    parser.add_argument(
        "--headless",
        help="Enable headless mode (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--delete_cookies",
        help="Delete cookies after quitting the driver (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    args = parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")

    return args


def collect_urls_arg_parser():
    """Provides a parser to parse arguments for the 
    collect_urls.py script.

    Returns:
        namespace, Parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Collect product-listing pages URLs data.")

    parser.add_argument(
        "--headless",
        help="Enable headless mode (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--delete_cookies",
        help="Delete cookies after quitting the driver (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--use_driver",
        help="Use the driver to get the page data (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--use_proxy",
        help="Use a proxy to get the page data (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--proxy_url",
        help="Proxy URL for seleniumwire options -> https://USERNAME:PASSWORD@HOST:PORT",
        type=str,
        default=None,
    )

    parser.add_argument(
        "--products_listing_page_url",
        help="Products-listing page URL.",
        type=str,
        default=False,
    )

    parser.add_argument(
        "--from_brands",
        help="Collect URLs from the products-listing page brands file (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--from_categories",
        help="Collect URLs from the products-listing page categories file (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--from_search_keywords",
        help="Collect URLs from the products-listing page search keywords file (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    args = parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")

    return args


def get_urls_object_name(args_urls_file_name, urls_folder_path):
    """Gets the filtered URLs file name to generate the URLs to collect.
    
    Args:
        args (dict): parsed arguments.
        urls_folder_path (str): path to filtered URLs folder.
        
    Returns:
        str, Filtered URLs file name.
    """

    if args_urls_file_name:
        urls_file_name = \
            os.path.join(urls_folder_path, args_urls_file_name)
    else:
        urls_file_name = \
            get_most_recent_json_file(folder_path=urls_folder_path)
    
    return urls_file_name


def generate_urls_to_collect_arg_parser():
    """Provides a parser to parse arguments for the 
    generate_urls_to_collect.py script.
        
    Returns:
        namespace, Parser with the parsed arguments.
    """
    
    parser = argparse.ArgumentParser(
        description="Generate URLs to collect.")

    parser.add_argument(
        "--n_parts", 
        help="Number of partitions to create.", 
        type=int, 
        default=1
    )

    parser.add_argument(
        "--filtered_urls_file_name", 
        help="Filtered URLs file name.", 
        type=str, 
        default=False
    )

    parser.add_argument(
        "--n_reviews_per_page", 
        help="Number of reviews per page.", 
        type=int, 
        default=False
    )

    args=parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")

    return args


def collect_page_arg_parser():
    """Provides a parser to parse arguments for the 
    collect_page.py script.

    Returns:
        namespace, Parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Collect page product data.")

    parser.add_argument(
        "--headless",
        help="Enable headless mode (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--delete_cookies",
        help="Delete cookies after quitting the driver (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--use_driver",
        help="Use the driver to get the page data (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--use_proxy",
        help="Use a proxy to get the page data (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--proxy_url",
        help="Proxy URL for seleniumwire options -> https://USERNAME:PASSWORD@HOST:PORT",
        type=str,
        default=None,
    )

    parser.add_argument(
        "--url",
        help="Product page URL.",
        type=str,
        default=False,
    )
    
    parser.add_argument(
        "--n_reviews_per_page", 
        help="Number of reviews per page.", 
        type=int, 
        default=False
    )

    parser.add_argument(
        "--n_max_reviews", 
        help="Max number of reviews to collect.", 
        type=int, 
        default=10000
    )

    parser.add_argument(
        "--min_date_year", 
        help="Minimum review date year to collect.", 
        type=int, 
        default=2000
    )
    
    args=parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")

    return args


def collect_pages_arg_parser():
    """Provides parser to parse arguments for the 
    collect_pages.py script.

    Returns:
        namespace, parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Collect product pages data.")

    parser.add_argument(
        "--headless",
        help="Enable headless mode (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=False
    )

    parser.add_argument(
        "--delete_cookies", 
        help="Delete cookies after quitting the driver (True/False)", 
        type=str_to_bool,
        choices=[True, False],
        default=False
    )
    
    parser.add_argument(
        "--use_driver",
        help="Use the driver to get the page data (True/False).",
        type=str_to_bool,
        choices=[True, False],
        default=True,
    )

    parser.add_argument(
        "--use_proxy",
        help="Proxy configuration for seleniumwire options.",
        type=str_to_bool,
        choices=[True, False],
        default=False,
    )

    parser.add_argument(
        "--proxy_url",
        help="Proxy URL for seleniumwire options -> https://USERNAME:PASSWORD@HOST:PORT",
        type=str,
        default=None,
    )

    parser.add_argument(
        "--urls_to_collect_file_name", 
        help="URLs to collect file name.", 
        type=str, 
        default=False
    )
    
    parser.add_argument(
        "--urls_to_collect_status", 
        help="Status of URLs to collect.", 
        type=str, 
        default="no"
    )
    
    parser.add_argument(
        "--prospected_urls_to_collect_status", 
        help="Status of prospected URLs to collect.", 
        type=str, 
        default="no"
    )

    parser.add_argument(
        "--reviews_urls_to_collect_status", 
        help="Status of reviews URLs to collect.", 
        type=str, 
        default="no"
    )

    parser.add_argument(
        "--n_reviews_per_page", 
        help="Number of reviews per page.", 
        type=int, 
        default=False
    )

    parser.add_argument(
        "--n_max_reviews", 
        help="Number of maximum reviews to collect.", 
        type=int, 
        default=10000
    )
    
    parser.add_argument(
        "--min_date_year", 
        help="Oldest review year to collect.", 
        type=int, 
        default=2000
    )
    
    args=parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")

    return args


def evaluate_collect_progression_arg_parser():
    """Provides parser to parse arguments for the 
    evaluate_collect_progression.py script.

    Returns:
        namespace, parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Evaluates collect progression.")
    
    parser.add_argument(
        "--urls_to_collect_file_name", 
        help="URLs to collect file name.", 
        type=str, 
        default=False)
    
    args=parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")
    
    return args


def transfer_files_to_s3_arg_parser():
    """Provides parser to parse arguments for the 
    transfer_files_to_s3.py script.

    Returns:
        namespace, parser with the parsed arguments.
    """

    parser = argparse.ArgumentParser(
        description="Transfer files to S3.")
    
    parser.add_argument(
        "--aggregated_urls", 
        help="Transfer aggregated URLs", 
        type=str_to_bool, 
        default=True)

    parser.add_argument(
        "--filtered_urls", 
        help="Transfer filtered URLs", 
        type=str_to_bool, 
        default=True)

    parser.add_argument(
        "--urls_to_collect", 
        help="Transfer URLs to collect", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--urls_to_collect_anchor", 
        help="Transfer URLs to collect anchor", 
        type=str_to_bool, 
        default=True)

    parser.add_argument(
        "--aggregated_products", 
        help="Transfer aggregated products", 
        type=str_to_bool, 
        default=True)

    parser.add_argument(
        "--aggregated_reviews", 
        help="Transfer aggregated reviews", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--counts_per_urls", 
        help="Transfer counts_per_urls KPIs files.", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--counts_per_collect_date_products", 
        help="Transfer counts_per_collect_date_products KPIs files.", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--counts_per_collect_date_reviews", 
        help="Transfer counts_per_collect_date_reviews KPIs files.", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--samples_products", 
        help="Transfer samples_products KPIs files.", 
        type=str_to_bool, 
        default=True)
    
    parser.add_argument(
        "--samples_reviews", 
        help="Transfer samples_reviews KPIs files.", 
        type=str_to_bool, 
        default=True)

    args=parser.parse_args()
    print(f"[LOG] Arguments parsed: {args}")
    
    return args
        


# !/usr/bin/env python

import time


def init_url_dict(source_dict):
    """Initializes the dictionary with the new URL data.

    Args:
        source_dict (dict): dictionary with information from the source.

    Returns:
       dict, Dictionary with the new URL data.
    """

    url_dict = {
        'id': None,
        # Product name & brand
        'product_name': None, # (str)
        'product_sub_name': None, # (str)
        'product_brand': None, # (str)
        'product_brand_line': None, # (str)
        # Product information
        'product_price': None, # (str)
        'product_type' : None, # (str)
        # Ratings
        'mean_rating': None, # (str / int / float)
        'n_reviews': 0, # (int)
        # Codes
        'code_asin': None, # (str)
        'code_ean': None, # (str)
        'code_gtin': None, # (str)
        'code_sku': None, # (str)
        'code_source': None, # (str)
        # URL
        'url': None,
        'images_urls': None, # (list)
        # Categories
        'category': None, # (str)
        'sub_category': None, # (str)
        'sub_sub_category': None, # (str)
        'sub_sub_sub_category': None, # (str)
        # Origin
        'products_listing_page_origin': None,
        'products_listing_page_url': None, 
        'products_listing_page_product_brand': None,
        'products_listing_page_search_keyword': None,
        'products_listing_page_category': None,
        'products_listing_page_sub_category': None,
        'products_listing_page_sub_sub_category': None, 
        # Source
        'country': source_dict['country'], # (str)
        'language': source_dict['language'], # (str)
        'source': source_dict['source'], # (str)
        # Collecte
        'collect_date': str(time.strftime('%Y-%m-%d')), # (str)
    }

    return url_dict


def init_product_dict(source_dict):
    """Initializes the dictionary with the product data.

    Args:
        source_dict (dict): dictionary with information from the source.

    Returns:
        dict, Dictionary with the product data.
    """

    product_dict = {
        # Product name & brand
        'product_name': None, # (str)
        'product_sub_name': None, # (str)
        'product_brand': None, # (str)
        'product_brand_line': None, # (str)
        # Product information
        'product_advice': None, # (str)
        'product_application_ld_json_dict': None, # (dict)
        'product_attribute': None, # (str)
        'product_attribute_dict': None, # (dict)
        'product_detail': None, # (str)
        'product_detail_dict': None, # (dict)
        'product_description': None, # (dict)
        'product_formulation': None, # (str)
        'product_information': None, # (str)
        'product_information_dict': None, # (dict)
        'product_price': None, # (str)
        'product_syndication': None, # (str)
        'product_syndication_dict': None, # (dict)
        'product_type' : None, # (str)
        # Ratings
        'mean_rating': None, # (str / int / float)
        'n_ratings_only': 0, # (int)
        'n_reviews': 0, # (int)
        'n_reviews_only': 0, # (int)
        'product_other_rating': None, # (str)
        'product_other_rating_dict': None, # (dict)
        # URL
        'url': None,
        'images_urls': None, # (list)
        # Categories
        'category': None, # (str)
        'sub_category': None, # (str)
        'sub_sub_category': None, # (str)
        'sub_sub_sub_category': None, # (str)
        # Codes
        'code_asin': None, # (str)
        'code_ean': None, # (str)
        'code_gtin': None, # (str)
        'code_sku': None, # (str)
        'code_source': None, # (str)
        # Source
        'country': source_dict['country'], # (str)
        'language': source_dict['language'], # (str)
        'source': source_dict['source'], # (str)
        # Collecte
        'collect_date': str(time.strftime('%Y-%m-%d')), # (str)
    }

    return product_dict


def init_review_dict(source_dict):
    """Initializes the dictionary with the review data.

    Returns:
        dict: Dictionary with the review data.
    """

    review_dict = {
        'id': None,
        # Product name & brand
        'product_name': None, # (str)
        'product_sub_name': None, # (str)
        'product_brand': None, # (str)
        'product_brand_line': None, # (str)
        'product_attribute': None, # (str)
        'product_attribute_dict': None, # (dict)
        # URL
        'url': None, # (str)
        'review_url': None, # (str)
        # Categories
        'category': None, # (str)
        'sub_category': None, # (str)
        'sub_sub_category': None, # (str)
        'sub_sub_sub_category': None, # (str)
        # Codes
        'code_asin': None, # (str)
        'code_ean': None, # (str)
        'code_gtin': None, # (str)
        'code_sku': None, # (str)
        'code_source': None, # (str)
        # Writer
        'writer_age': None, # (str)
        'writer_information': None, # (str)
        'writer_information_dict': None, # (dict)
        'writer_location': None, # (str)
        'writer_pseudo': None, # (str)
        'writer_recommendation': None, # (str)
        'writer_sex': None, # (str)
        'writer_usage': None, # (str)
        # Review
        'review_rating': None, # (str)
        'review_other_rating': None, # (str)
        'review_other_rating_dict': None, # (dict)
        'review_date': None, # (str)
        'review_date_appended': None, # (str)
        'review_title': None, # (str)
        'review_text': None, # (str)
        'review_text_appended': None, # (str)
        'review_text_strength': None, # (str)
        'review_text_weakness': None, # (str)
        # Labels
        'incentive': None, # (str)
        'syndication': None, # (str)
        'utility': None, # (str)
        'utility_yes': None, # (str)
        'utility_no': None, # (str)
        'verified_purchase': None, # (str)
        # Source
        'country': source_dict['country'], # (str)
        'language': source_dict['language'], # (str)
        'source': source_dict['source'], # (str)
        # Collecte
        'collect_date': str(time.strftime('%Y-%m-%d')), # (str)
    }

    return review_dict
